#ifndef GRAPH_REF_HPP
#define GRAPH_REF_HPP

#include <utility>

template<class T>
class graph_ref;

template<class T>
std::pair<typename T::edge_descriptor, bool> 
add_edge(typename T::vertex_descriptor t, 
	 typename T::vertex_descriptor h, 
	 graph_ref<T> &g) {
  return boost::add_edge(t,h,g._g);
}

template<class T>
typename T::vertices_size_type num_vertices(graph_ref<T> const &g) {
  return boost::num_vertices(g);
}

template<class T, class PropertyTag>
typename boost::property_map<graph_ref<T>, PropertyTag>::type 
get(PropertyTag tag, graph_ref<T> &g) { 
  return boost::get(tag, g._g);
}


template<class T>
class graph_ref {
private:
  typedef graph_ref<T> self;
public:
  // typedefs
  typedef typename T::vertex_descriptor vertex_descriptor;
  typedef typename T::edge_descriptor edge_descriptor;
  typedef typename T::vertices_size_type vertices_size_type;
  typedef typename T::edges_size_type edges_size_type;
  typedef typename T::degree_size_type degree_size_type;
  typedef typename T::out_edge_iterator out_edge_iterator;
  typedef typename T::in_edge_iterator in_edge_iterator;
  typedef typename T::adjacency_iterator adjacency_iterator;
  typedef typename T::edge_iterator edge_iterator;
  typedef typename T::vertex_iterator vertex_iterator;
  typedef typename T::vertex_property_type vertex_property_type;
  typedef typename T::edge_property_type edge_property_type;
  typedef typename T::graph_tag graph_tag;
  typedef typename T::edge_parallel_category edge_parallel_category;
  typedef typename T::directed_category directed_category;
  typedef typename T::traversal_category traversal_category;

  // friend functions

  friend std::pair<typename T::edge_descriptor, bool> 
  add_edge<T>(typename T::vertex_descriptor, 
	      typename T::vertex_descriptor, 
	      self &);  

  friend typename T::vertices_size_type 
  num_vertices<>(graph_ref<T> const &);

  template<class P> friend typename boost::property_map<graph_ref<T>, P>::type get(P, graph_ref<T> &);
private:
  T &_g;
public:
  graph_ref(T &g) : _g(g) {}
};

#endif




