#ifndef CONDENSATION_GRAPH_HPP
#define CONDENSATION_GRAPH_HPP

#include <vector>
#include <boost/counting_iterator.hpp>

using namespace std;

template<class T>
class condensation_graph;

template<class T>
std::pair<typename condensation_graph<T>::component_iterator,
	  typename condensation_graph<T>::component_iterator> 
components(condensation_graph<T> const &g) {
  return std::make_pair(typename condensation_graph<T>::component_iterator(0),
			typename condensation_graph<T>::component_iterator(g._comps.size()));
}

template<class T>
typename condensation_graph<T>::component_descriptor
add_component(condensation_graph<T> &g) {
  typename condensation_graph<T>::component_descriptor r(g._comps.size());
  g._comps.push_back(vector<unsigned int>());
  return r;
}

template<class T>
void insert(typename T::vertex_descriptor n,
	    typename condensation_graph<T>::component_descriptor c, 
	    condensation_graph<T> &g) {
  g._comps[c].push_back(n);
}

template<class T>
class condensation_graph_iterator {
private:
  unsigned int _eno;
  condensation_graph<T> *_parent;
public:
  condensation_graph_iterator(unsigned int n = 0,
			      condensation_graph<T> *p = NULL) 
    : _eno(n), _parent(p) {
  }

  condensation_graph_iterator operator=(condensation_graph_iterator const &src) {
    _eno=src._eno;
    _parent=src._parent;
  }

  bool operator==(condensation_graph_iterator const &src) const {
    return src._eno == _eno;
  }

  void operator++(int) { ++_eno; }
  
  condensation_graph_iterator operator++(void) { 
    condensation_graph_iterator r(*this);
    ++_eno;
    return *this;
  }
  
  typename T::vertex_descriptor operator*(void) {
    return _parent->_get_vd(_eno);
  }
};

template<class T>
std::pair<typename condensation_graph<T>::component_out_edge_iterator,
	 typename condensation_graph<T>::component_out_edge_iterator>
out_edges(typename condensation_graph<T>::component_descriptor c,
	  condensation_graph<T> &g) {
  return std::make_pair(typename condensation_graph<T>::component_out_edge_iterator(0,&g),
			typename condensation_graph<T>::component_out_edge_iterator(g._get_vd_count(),&g));
}

template<class T>
class condensation_graph : public T {
public:
  typedef unsigned int component_descriptor;
  typedef boost::counting_iterator_generator<unsigned int>::type component_iterator;
  typedef condensation_graph_iterator<T> component_out_edge_iterator;
  
  friend std::pair<component_iterator,component_iterator> components<T>(condensation_graph const &);
  friend condensation_graph<T>::component_descriptor add_component<T>(condensation_graph &);
  friend void insert<T>(typename T::vertex_descriptor, component_descriptor, condensation_graph &);
  friend std::pair<component_out_edge_iterator,component_out_edge_iterator> out_edges<T>(component_descriptor c,
										     condensation_graph &g);
  friend class condensation_graph_iterator<T>;  
private:
  vector<vector<typename T::vertex_descriptor> > _comps; // set of components
public:
  condensation_graph(unsigned int n) 
    : T(n) {
  }

  condensation_graph(T const &g) 
    : T(g) {
  }
private:
  typename T::vertex_descriptor _get_vd(unsigned int n) {
    return _comps[0][0];
  }
  unsigned int _get_vd_count(void) {
    return 0;
  }
};

#endif
